(self.webpackChunkvp4jl=self.webpackChunkvp4jl||[]).push([[86],{624:t=>{function e(t){if(t)return function(t){for(var o in e.prototype)t[o]=e.prototype[o];return t}(t)}t.exports=e,e.prototype.on=e.prototype.addEventListener=function(t,e){return this._callbacks=this._callbacks||{},(this._callbacks[t]=this._callbacks[t]||[]).push(e),this},e.prototype.once=function(t,e){var o=this;function n(){o.off(t,n),e.apply(this,arguments)}return this._callbacks=this._callbacks||{},n.fn=e,this.on(t,n),this},e.prototype.off=e.prototype.removeListener=e.prototype.removeAllListeners=e.prototype.removeEventListener=function(t,e){if(this._callbacks=this._callbacks||{},0==arguments.length)return this._callbacks={},this;var o,n=this._callbacks[t];if(!n)return this;if(1==arguments.length)return delete this._callbacks[t],this;for(var r=0;r<n.length;r++)if((o=n[r])===e||o.fn===e){n.splice(r,1);break}return this},e.prototype.emit=function(t){this._callbacks=this._callbacks||{};var e=[].slice.call(arguments,1),o=this._callbacks[t];if(o)for(var n=0,r=(o=o.slice(0)).length;n<r;++n)o[n].apply(this,e);return this},e.prototype.listeners=function(t){return this._callbacks=this._callbacks||{},this._callbacks[t]||[]},e.prototype.hasListeners=function(t){return!!this.listeners(t).length}},3086:(t,e,o)=>{var n=o(624);function r(){n.call(this)}r.prototype=new n,t.exports=r,r.Stream=r,r.prototype.pipe=function(t,e){var o=this;function n(e){t.writable&&!1===t.write(e)&&o.pause&&o.pause()}function r(){o.readable&&o.resume&&o.resume()}o.on("data",n),t.on("drain",r),t._isStdio||e&&!1===e.end||(o.on("end",i),o.on("close",c));var s=!1;function i(){s||(s=!0,t.end())}function c(){s||(s=!0,"function"==typeof t.destroy&&t.destroy())}function a(t){if(l(),!this.hasListeners("error"))throw t}function l(){o.off("data",n),t.off("drain",r),o.off("end",i),o.off("close",c),o.off("error",a),t.off("error",a),o.off("end",l),o.off("close",l),t.off("end",l),t.off("close",l)}return o.on("error",a),t.on("error",a),o.on("end",l),o.on("close",l),t.on("end",l),t.on("close",l),t.emit("pipe",o),t}}}]);