export namespace vp4jlIDs {
  export const fileExtension = '.vp4jl';
  export const fileType = 'vp4jl';
  export const trackerNamespace = 'vp4jl';
  export const widgetFactory = 'vp4jl widget factory';
  export const modelFactory = 'vp4jl model factory';
  export const createNewLabelInPalette = 'New Visual Programming File';
  export const createNewLabelInFileMenu = 'Visual Programming File';
  export const createNewLabelInContextMenu = 'Visual Programming File';
  export const caption = 'Create a new Visual Programming file';
  export const nodeExtension = 'vp4jl-node-extension';
}

export namespace vp4jlCommandIDs {
  export const commandCategory = 'Visual Programming';
  export const createNew = 'vp4jl:create-new';
  export const run = 'vp4jl:run';
  export const copy = 'vp4jl:copy';
  export const paste = 'vp4jl:paste';
  export const del = 'vp4jl:delete';
  export const cut = 'vp4jl:cut';
  export const duplicate = 'vp4jl:duplicate';
  export const deleteAll = 'vp4jl:clear';
  export const clearOutput = 'vp4jl:clear-output';
  export const shutdownKernel = 'vp4jl:shutdown-kernel';
  export const interruptKernel = 'vp4jl:interrupt-kernel';
  export const restartKernel = 'vp4jl:restart-kernel';
  export const reconnectKernel = 'vp4jl:reconnect-kernel';
  export const changeKernel = 'vp4jl:change-kernel';
  export const restartKernelAndRun = 'vp4jl:restart-kernel-and-run';
  export const showNodeExtension = 'vp4jl:show-node-extension';
  export const hideNodeExtension = 'vp4jl:hide-node-extension';
  export const toggleNodeExtension = 'vp4jl:toggle-node-extension';
  export const toggleOutput = 'vp4jl:toggle-output';
}
